// SPDX-License-Identifier: GPL-2.0
/*
 * Driver for FPGA Management Engine Ethernet Group
 *
 * Copyright 2018-2019 Intel Corporation, Inc.
 *
 */
#include <linux/intel-fpga.h>
#include <linux/pac_n3000_net.h>

#include "feature-dev.h"
#include "fme.h"

static int eth_group_init(struct platform_device *pdev, struct feature *feature)
{
	struct feature_fme_header *fme_hdr
		= get_feature_ioaddr_by_id(&pdev->dev, FME_FEATURE_ID_HEADER);
	struct feature_fme_bitstream_id bitstream_id;
	struct pac_n3000_net_platdata pdata;
	struct platform_device *subdev;

	dev_dbg(&pdev->dev, "FME ETH GROUP Init\n");

	memset(&pdata, 0, sizeof(pdata));

	bitstream_id.csr = readq(&fme_hdr->bitstream_id);

	pdata.config = bitstream_id.hssi_id;
	pdata.base = feature->ioaddr;
	pdata.parent_id = pdev->id;

	subdev = feature_create_subdev(pdev, PAC_N3000_NET_DRV, &pdata,
				       sizeof(pdata));
	if (IS_ERR(subdev))
		return PTR_ERR(subdev);

	feature_set_priv(feature, subdev);
	return 0;
}

static void eth_group_uinit(struct platform_device *pdev,
			    struct feature *feature)
{
	struct platform_device *subdev = feature_get_priv(feature);

	dev_dbg(&pdev->dev, "FME ETH GROUP UInit\n");

	feature_destroy_subdev(subdev);
	feature_set_priv(feature, NULL);
}

struct feature_ops eth_group_ops = {
	.init = eth_group_init,
	.uinit = eth_group_uinit,
};
