/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Header File for Intel Programmable Acceleration Card HSSI Configurable
 * Ethernet Driver
 *
 * Copyright 2018-2019 Intel Corporation, Inc.
 *
 */
#ifndef __INTEL_PAC_HSSI_H
#define __INTEL_PAC_HSSI_H

#include <linux/device.h>

#define PAC_HSSI_DRV_NAME	"intel-pac-hssi"

/* sysfs config values to specify ethernet frequency */
#define HSSI_ETH_SPEED_10G	10
#define HSSI_ETH_SPEED_40G	40

#define I2C_SERIAL_NUMBER_LEN	16	/* 16 bytes / 128 bits */
#define I2C_EEPROM_LEN		512	/* 512 bytes in EEPROM */

#define HSSI_WRITE_POLL_INVL_US		10	/* Write poll interval */
#define HSSI_WRITE_POLL_TIMEOUT_US	100000	/* Write poll timeout */

int hssi2_create_sysfs(struct device *dev);
void hssi2_remove_sysfs(struct device *dev);

struct pac_hssi_plat_data {
	void __iomem *csr_base;
	u8 revision;
};

struct pac_hssi {
	void __iomem *csr_base;
	struct device *dev;
	struct mutex bus_mutex;
	u8 revision;
	u8 eeprom[I2C_EEPROM_LEN];		 /* Cache eeprom data */
	u8 serial_number[I2C_SERIAL_NUMBER_LEN]; /* Cache serial number */
	u8 data_flags;
	struct list_head qobj_list;
};

#endif /* __INTEL_PAC_HSSI_H */
