/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Driver Header File for Intel SPI Slave to AVMM Bus Bridge
 *
 * Copyright (C) 2018 Intel Corporation. All rights reserved.
 *
 */

#ifndef __INTEL_SPI_AVMM_H
#define __INTEL_SPI_AVMM_H

#include <linux/spi/spi.h>
#include <linux/regmap.h>

/**
 * struct spi_avmm_bridge - SPI slave to AVMM bus master bridge
 *
 * @spi: spi slave associated with this bridge.
 * @lock: lock to protect transactions over this bridge.
 *
 * As device's registers are implemented on the AVMM bus memory space, it
 * requres driver to issue formatted requests to spi slave to AVMM bus master
 * bridge to perform register access.
 */
struct spi_avmm_bridge {
	struct spi_device *spi;
	struct mutex lock;

	/* per bridge buffer ptrs used in translation between protocol layers */
	char *trans_tx_buf;
	char *trans_rx_buf;
	char *packet_tx_buf;
	char *packet_rx_buf;
	char *phy_tx_buf;
	char *phy_rx_buf;
};

struct spi_avmm_bridge *spi_avmm_bridge_alloc_and_init(struct spi_device *spi);
void spi_avmm_bridge_destroy(struct spi_avmm_bridge *bridge);

struct regmap_spi_avmm_ctx {
	struct spi_avmm_bridge *bridge;
	unsigned int regoff;
	size_t size;
};

struct regmap_spi_avmm_ctx *
regmap_spi_avmm_gen_ctx(struct spi_avmm_bridge *bridge, unsigned int regoff,
			size_t size);

/* for full memory space */
#define regmap_spi_avmm_gen_ctx_full(bridge) \
	regmap_spi_avmm_gen_ctx(bridge, 0, 0)

extern struct regmap_bus regmap_spi_avmm_bus;

#endif /* __REGMAP_SPI_AVMM_H */
