/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Header file for Intel FPGA Security Manager
 *
 *  Copyright (C) 2019 Intel Corporation, Inc
 */
#ifndef _LINUX_IFPGA_SEC_MGR_H
#define _LINUX_IFPGA_SEC_MGR_H

#include <linux/cdev.h>
#include <uapi/linux/ifpga-sec-mgr.h>

struct ifpga_sec_mgr;

struct ifpga_sec_mgr_ops {
	int (*start)(struct ifpga_sec_mgr *imgr);
	int (*write_blk)(struct ifpga_sec_mgr *imgr,
			 struct ifpga_secure_write *write_blk, char *buf);
	int (*data_sent)(struct ifpga_sec_mgr *imgr);
	int (*check_complete)(struct ifpga_sec_mgr *imgr);
	int (*cancel)(struct ifpga_sec_mgr *imgr);
};

struct ifpga_sec_mgr {
	const char *name;
	struct device dev;
	struct cdev cdev;
	const struct ifpga_sec_mgr_ops *iops;
	struct mutex lock;
	unsigned long opened;
	void *priv;
};

int ifpga_sec_mgr_register(struct ifpga_sec_mgr *imgr, struct module *owner);
void ifpga_sec_mgr_unregister(struct ifpga_sec_mgr *imgr);
struct ifpga_sec_mgr *ifpga_sec_mgr_create(struct device *dev, const char *name,
					   const struct ifpga_sec_mgr_ops *iops,
					   void *priv);
void ifpga_sec_mgr_free(struct ifpga_sec_mgr *imgr);

#endif
