/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Header File for I2C Altera Controller Driver.
 *
 * Copyright Intel Corporation (C) 2018.
 */
#ifndef __LINUX_I2C_ALTERA_H
#define __LINUX_I2C_ALTERA_H

#include <linux/clk.h>
#include <linux/i2c.h>
#include <linux/types.h>
#include <linux/regmap.h>

#define ALTERA_I2C_DRV_NAME	"altera-i2c"

/**
 * struct altr_i2c_platform_data - Platform data of the Altera I2C driver
 * @num_devices:	Number of devices that shall be added when the driver
 *			is probed.
 * @devices:		The actuall devices to add.
 * @fifo_size:		Size of the FIFO passed in.
 * @bus_clk_rate:	I2C bus clock rate
 * @i2c_clk:		Input clock.
 * @regbus:		regbus used for regmap_init().
 * @regmap_ctx:		context used for regmap_init().
 *
 * This purpose of this platform data struct is to be able to provide a number
 * of devices that should be added to the I2C bus. The reason is that sometimes
 * the I2C board info is not enough, a new PCI board can for instance be
 * plugged into a standard PC, and the bus number might be unknown at
 * early init time.
 */
struct altr_i2c_platform_data {
	u8				num_devices;
	struct i2c_board_info const	*devices;
	u32				fifo_size;
	u32				bus_clk_rate;
	struct clk			*i2c_clk;
	struct regmap_bus		regbus;
	void				*regmap_ctx;
};

#endif /* __LINUX_I2C_ALTERA_H */
