/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Intel(R) PAC N3000 Network Driver Header File
 *
 * Copyright (C) 2019 Intel Corporation. All rights reserved.
 */

#ifndef __PAC_N3000_NET_H__
#define __PAC_N3000_NET_H__

#define PAC_N3000_NET_DRV "pac_n3000_net"

/**
 * struct pac_n3000_net_platdata - platform data for pac n3000 network device
 *
 * @base: mapped base address of register region.
 * @parent_id: unique id of parent device.
 * @config: network configuration of pac n3000
 */
struct pac_n3000_net_platdata {
	void __iomem *base;
	unsigned int parent_id;
	unsigned int config;
};

#define ETH_CONFIG_8x10G	0
#define ETH_CONFIG_4x25G	1
#define ETH_CONFIG_2x1x25G	2
#define ETH_CONFIG_4x25G_2x25G  3
#define ETH_CONFIG_2x2x25G	4

/**
 * struct pkvl_dev - pkvl device data structure
 *
 * @parent_id: unique id of parenet device.
 * @id: unique id of pkvl device.
 * @link_id: pkvl link index.
 * @node: used to link to global pvdev_list.
 * @netdev: net device.
 * @get_link: ops to get link status.
 */
struct pkvl_dev {
	unsigned int parent_id;
	unsigned int id;
	unsigned int link_id;
	struct list_head node;
	struct net_device *netdev;
	void *priv;
	u32 (*get_link)(struct pkvl_dev *pvdev);
};

int register_pkvl_dev(struct pkvl_dev *pvdev);
void unregister_pkvl_dev(struct pkvl_dev *pvdev);

#endif /* __PAC_N3000_NET_H__ */
