/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Header File for SPI Altera Controller Driver.
 */
#ifndef __LINUX_SPI_ALTERA_H
#define __LINUX_SPI_ALTERA_H

#include <linux/spi/spi.h>
#include <linux/types.h>
#include <linux/regmap.h>

#define ALTERA_SPI_DRV_NAME	"spi-altera"

/*
 * Later kernel versions include SPI_BIT_MASK() and SPI_BPW_RANGE_MASK()
 * definitions in linux/spi.h. These definitions are here for older
 * (e.g. 3.10) kernels.
 */
#ifndef SPI_BIT_MASK
#define SPI_BIT_MASK(bits) (((bits) == 32) ? ~0U : (BIT(bits) - 1))
#endif /* SPI_BIT_MASK */

#ifndef SPI_BPW_RANGE_MASK
#define SPI_BPW_RANGE_MASK(min, max) (SPI_BIT_MASK(max) - SPI_BIT_MASK(min - 1))
#endif /* SPI_BPW_RANGE_MASK */

/**
 * struct altera_spi_platform_data - Platform data of the Altera SPI driver
 * @mode_bits:		Mode bits of SPI master.
 * @num_chipselect:	Num of chipselects.
 * @bits_per_word_mask: bitmask of supported bits_per_word for transfers.
 * @num_devices:	Number of devices that shall be added when the driver
 *			is probed.
 * @devices:		The actuall devices to add.
 * @regbus:		regbus used for regmap_init().
 * @regmap_ctx:		context used for regmap_init().
 *
 * This purpose of this platform data struct is to be able to provide a number
 * of devices that should be added to the SPI bus. The reason is that sometimes
 * the SPI board info is not enough, a new SPI board can for instance be
 * plugged into a standard PC, and the bus number might be unknown at
 * early init time.
 */
struct altera_spi_platform_data {
	u16				mode_bits;
	u16				num_chipselect;
	u32				bits_per_word_mask;
	u8				num_devices;
	struct spi_board_info		*devices;
	struct regmap_bus		regbus;
	void				*regmap_ctx;
};

#endif /* __LINUX_SPI_ALTERA_H */
