/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 * Header File for FPGA DFL User API
 *
 * Copyright (C) 2019 Intel Corporation, Inc.
 *
 */

#ifndef _UAPI_LINUX_IFPGA_SEC_MGR_H
#define _UAPI_LINUX_IFPGA_SEC_MGR_H

#include <linux/types.h>
#include <linux/ioctl.h>

#define IFPGA_SEC_MGR_MAGIC 0xB9

#define IFPGA_SECURE_UPDATE_START	_IO(IFPGA_SEC_MGR_MAGIC, 0)

/**
 * IFPGA_SECURE_UPDATE_WRITE_BLK - _IOW(IFPGA_SEC_MGR_MAGIC, 1,
 *					struct ifpga_secure_write);
 *
 * Write a block of data into the flash staging area.
 * The user must provide the offset and the size of
 * the data block.
 * Return: 0 on success, -errno on failure.
 */
struct ifpga_secure_write {
	/* Input */
	__u32 argsz;		/* Structure length */
	__u32 flags;		/* Zero for now */
	__u32 offset;		/* Destination offset in staging area */
	__u32 size;		/* Data size (in bytes) to be written */
	__u64 buf;		/* User space address of source data */
};

#define IFPGA_SECURE_UPDATE_WRITE_BLK		_IO(IFPGA_SEC_MGR_MAGIC, 1)
#define IFPGA_SECURE_UPDATE_DATA_SENT		_IO(IFPGA_SEC_MGR_MAGIC, 2)
#define IFPGA_SECURE_UPDATE_CHECK_COMPLETE	_IO(IFPGA_SEC_MGR_MAGIC, 3)
#define IFPGA_SECURE_UPDATE_CANCEL		_IO(IFPGA_SEC_MGR_MAGIC, 4)

#endif
