/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 * Header File for Intel(R) PAC N3000 User API
 *
 * Copyright (C) 2019 Intel Corporation. All rights reserved.
 */

#ifndef _UAPI_PAC_N3000_NET_H
#define _UAPI_PAC_N3000_NET_H

#include <linux/types.h>

#define ETH_GROUP_MAGIC 0xB7

#define ETH_GROUP_API_VERSION 0

/**
 * ETH_GROUP_GET_API_VERSION - _IO(ETH_GROUP_MAGIC, 0)
 *
 * Report the version of the driver API.
 * Return: Driver API Version.
 */

#define ETH_GROUP_GET_API_VERSION	_IO(ETH_GROUP_MAGIC, 0)

/**
 * ETH_GROUP_CHECK_EXTENSION - _IO(ETH_GROUP_MAGIC, 1)
 *
 * Check whether an extension is supported.
 * Return: 0 if not supported, otherwise the extension is supported.
 */

#define ETH_GROUP_CHECK_EXTENSION	_IO(ETH_GROUP_MAGIC, 1)

/**
 * ETH_GROUP_GET_INFO - _IOR(ETH_GROUP_MAGIC, 2, struct eth_group_info)
 *
 * Retrieve information about the Eth Group.
 * Driver fills the info in provided struct eth_group_info.
 *
 * Return: 0 on success, -errno on failure.
 */
struct eth_group_info {
	/* Input */
	__u32 argsz;
	/* Output */
	__u32 flags;
#define ETH_GROUP_FLAGS_LW_MAC	(1 << 0)
	__u8 speed;		/* Eth Speed */
	__u8 phy_num;		/* Number of PHYs */
	__u8 mac_num;		/* Number of MACs */
	__u8 group_id;		/* Eth Group Id */
};

#define ETH_GROUP_GET_INFO		_IO(ETH_GROUP_MAGIC, 2)

/**
 * ETH_GROUP_READ_REG - _IOR(ETH_GROUP_MAGIC, 3, struct eth_group_read_reg)
 *
 * Read PHY, MAC or EthernetWrapper Register from ETH GROUP.
 *
 * Set ETH_GROUP_SELECT_FEAT flag, when needs to access additional feature
 * registers for this IP.
 *
 * Return: 0 on success, -errno on failure.
 */
struct eth_group_read_reg {
	/* Input */
	__u32 argsz;
	__u32 flags;
#define ETH_GROUP_SELECT_FEAT	(1 << 0)
	__u8 type;
#define ETH_GROUP_PHY		1
#define ETH_GROUP_MAC		2
#define ETH_GROUP_ETHER		3
	__u8 index;		/* Index of PHY, MAC or Ethernet Wrapper */
	__u16 addr;		/* Register address */
	/* Output */
	__u32 data;		/* Register value */
};

#define ETH_GROUP_READ_REG		_IO(ETH_GROUP_MAGIC, 3)

/**
 * ETH_GROUP_WRITE_REG - _IOR(ETH_GROUP_MAGIC, 4, struct eth_group_write_reg)
 *
 * Write PHY, MAC or EthernetWrapper Register from ETH GROUP.
 *
 * Set ETH_GROUP_SELECT_FEAT flag, when needs to access additional feature
 * registers for this IP.
 *
 * Return: 0 on success, -errno on failure.
 */
struct eth_group_write_reg {
	/* Input */
	__u32 argsz;
	__u32 flags;		/* Same as Read */
	__u8 type;		/* Same as Read */
	__u8 index;		/* PHY/MAC/EthernetWrapper index */
	__u16 addr;		/* Register address */
	__u32 data;		/* Data to write */
};

#define ETH_GROUP_WRITE_REG		_IO(ETH_GROUP_MAGIC, 4)

#endif /* __UAPI_PAC_N3000_NET_H */
